@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009, 2010, 2011 Nicolas Bonnefon and other contributors
+ * Copyright (C) 2009, 2010, 2011, 2012, 2013 Nicolas Bonnefon and other contributors
  *
  * This file is part of glogg.
  *
@@ -24,6 +24,7 @@
 // pattern.
 
 #include <iostream>
+#include <cassert>
 
 #include <QApplication>
 #include <QClipboard>
@@ -35,6 +36,7 @@
 #include <QScrollBar>
 #include <QMenu>
 #include <QAction>
+#include <QtCore>
 
 #include "log.h"
 
@@ -46,6 +48,48 @@
 #include "overview.h"
 #include "configuration.h"
 
+namespace {
+
+int countDigits( quint64 n )
+{
+    if (n == 0)
+        return 1;
+
+    // We must force the compiler to not store intermediate results
+    // in registers because this causes incorrect result on some
+    // systems under optimizations level >0. For the skeptical:
+    //
+    // #include <math.h>
+    // #include <stdlib.h>
+    // int main(int argc, char **argv) {
+    //     (void)argc;
+    //     long long int n = atoll(argv[1]);
+    //     return floor( log( n ) / log( 10 ) + 1 );
+    // }
+    //
+    // This is on Thinkpad T60 (Genuine Intel(R) CPU T2300).
+    // $ g++ --version
+    // g++ (Ubuntu/Linaro 4.6.3-1ubuntu5) 4.6.3
+    // $ g++ -O0 -Wall -W -o math math.cpp -lm; ./math 10; echo $?
+    // 2
+    // $ g++ -O1 -Wall -W -o math math.cpp -lm; ./math 10; echo $?
+    // 1
+    //
+    // A fix is to (1) explicitly place intermediate results in
+    // variables *and* (2) [A] mark them as 'volatile', or [B] pass
+    // -ffloat-store to g++ (note that approach [A] is more portable).
+
+    volatile qreal ln_n  = qLn( n );
+    volatile qreal ln_10 = qLn( 10 );
+    volatile qreal lg_n = ln_n / ln_10;
+    volatile qreal lg_n_1 = lg_n + 1;
+    volatile qreal fl_lg_n_1 = qFloor( lg_n_1 );
+
+    return fl_lg_n_1;
+}
+
+} // anon namespace
+
 
 LineChunk::LineChunk( int first_col, int last_col, ChunkType type )
 {
@@ -105,7 +149,9 @@ inline void LineDrawer::addChunk( const LineChunk& chunk,
 }
 
 inline void LineDrawer::draw( QPainter& painter,
-        int xPos, int yPos, int line_width, const QString& line )
+        int initialXPos, int initialYPos,
+        int line_width, const QString& line,
+        int leftExtraBackgroundPx )
 {
     QFontMetrics fm = painter.fontMetrics();
     const int fontHeight = fm.height();
@@ -114,13 +160,26 @@ inline void LineDrawer::draw( QPainter& painter,
     // following give the right result, not sure why:
     const int fontWidth = fm.width( QChar('a') );
 
+    int xPos = initialXPos;
+    int yPos = initialYPos;
+
     foreach ( Chunk chunk, list ) {
         // Draw each chunk
         // LOG(logDEBUG) << "Chunk: " << chunk.start() << " " << chunk.length();
         QString cutline = line.mid( chunk.start(), chunk.length() );
         const int chunk_width = cutline.length() * fontWidth;
-        painter.fillRect( xPos, yPos, chunk_width,
-                fontHeight, chunk.backColor() );
+        if ( xPos == initialXPos ) {
+            // First chunk, we extend the left background a bit,
+            // it looks prettier.
+            painter.fillRect( xPos - leftExtraBackgroundPx, yPos,
+                    chunk_width + leftExtraBackgroundPx,
+                    fontHeight, chunk.backColor() );
+        }
+        else {
+            // other chunks...
+            painter.fillRect( xPos, yPos, chunk_width,
+                    fontHeight, chunk.backColor() );
+        }
         painter.setPen( chunk.foreColor() );
         painter.drawText( xPos, yPos + fontAscent, cutline );
         xPos += chunk_width;
@@ -174,23 +233,18 @@ void DigitsBuffer::timerEvent( QTimerEvent* event )
 }
 
 // Graphic parameters
-
-// Looks better with an odd value
-const int AbstractLogView::bulletLineX_ = 11;
-const int AbstractLogView::leftMarginPx_ = bulletLineX_ + 2;
-
 const int AbstractLogView::OVERVIEW_WIDTH = 27;
 
 AbstractLogView::AbstractLogView(const AbstractLogData* newLogData,
         const QuickFindPattern* const quickFindPattern, QWidget* parent) :
     QAbstractScrollArea( parent ),
+    lineNumbersVisible_( false ),
     selectionStartPos_(),
     selectionCurrentEndPos_(),
     autoScrollTimer_(),
     selection_(),
     quickFindPattern_( quickFindPattern ),
-    quickFind_( newLogData, &selection_, quickFindPattern ),
-    overviewWidget_( this )
+    quickFind_( newLogData, &selection_, quickFindPattern )
 {
     logData = newLogData;
 
@@ -204,6 +258,11 @@ AbstractLogView::AbstractLogView(const AbstractLogData* newLogData,
     firstCol = 0;
 
     overview_ = NULL;
+    overviewWidget_ = NULL;
+
+    // Display
+    nbDigitsInLineNumber_ = 0;
+    leftMarginPx_ = 0;
 
     // Fonts
     charWidth_ = 1;
@@ -214,6 +273,10 @@ AbstractLogView::AbstractLogView(const AbstractLogData* newLogData,
 
     setAttribute( Qt::WA_StaticContents );  // Does it work?
 
+    // Hovering
+    setMouseTracking( true );
+    lastHoveredLine_ = -1;
+
     // Init the popup menu
     createMenu();
 
@@ -222,14 +285,10 @@ AbstractLogView::AbstractLogView(const AbstractLogData* newLogData,
             this, SLOT ( handlePatternUpdated() ) );
     connect( verticalScrollBar(), SIGNAL( sliderMoved( int ) ),
             this, SIGNAL( followDisabled() ) );
-    connect( &quickFind_, SIGNAL( notify( const QString& ) ),
-            this, SIGNAL( notifyQuickFind( const QString& ) ) );
+    connect( &quickFind_, SIGNAL( notify( const QFNotification& ) ),
+            this, SIGNAL( notifyQuickFind( const QFNotification& ) ) );
     connect( &quickFind_, SIGNAL( clearNotification() ),
             this, SIGNAL( clearQuickFindNotification() ) );
-    connect( &overviewWidget_, SIGNAL( lineClicked ( int ) ),
-            this, SIGNAL( followDisabled() ) );
-    connect( &overviewWidget_, SIGNAL( lineClicked ( int ) ),
-            this, SLOT( jumpToLine( int ) ) );
 }
 
 AbstractLogView::~AbstractLogView()
@@ -257,29 +316,37 @@ void AbstractLogView::mousePressEvent( QMouseEvent* mouseEvent )
 {
     static Configuration& config = Persistent<Configuration>( "settings" );
 
-    // Selection implementation
     if ( mouseEvent->button() == Qt::LeftButton )
     {
         int line = convertCoordToLine( mouseEvent->y() );
 
-        if ( mouseEvent->x() < leftMarginPx_ ) {
-            // Mark a line if it is clicked in the left margin
-            // (only if click and release in the same area)
-            markingClickInitiated_ = true;
-            markingClickLine_ = line;
+        if ( mouseEvent->modifiers() & Qt::ShiftModifier )
+        {
+            selection_.selectRangeFromPrevious( line );
+            emit updateLineNumber( line );
+            update();
         }
-        else {
-            // Select the line, and start a selection
-            if ( line < logData->getNbLine() ) {
-                selection_.selectLine( line );
-                emit updateLineNumber( line );
-                emit newSelection( line );
+        else
+        {
+            if ( mouseEvent->x() < bulletZoneWidthPx_ ) {
+                // Mark a line if it is clicked in the left margin
+                // (only if click and release in the same area)
+                markingClickInitiated_ = true;
+                markingClickLine_ = line;
             }
+            else {
+                // Select the line, and start a selection
+                if ( line < logData->getNbLine() ) {
+                    selection_.selectLine( line );
+                    emit updateLineNumber( line );
+                    emit newSelection( line );
+                }
 
-            // Remember the click in case we're starting a selection
-            selectionStarted_ = true;
-            selectionStartPos_ = convertCoordToFilePos( mouseEvent->pos() );
-            selectionCurrentEndPos_ = selectionStartPos_;
+                // Remember the click in case we're starting a selection
+                selectionStarted_ = true;
+                selectionStartPos_ = convertCoordToFilePos( mouseEvent->pos() );
+                selectionCurrentEndPos_ = selectionStartPos_;
+            }
         }
     }
     else if ( mouseEvent->button() == Qt::RightButton )
@@ -359,6 +426,9 @@ void AbstractLogView::mouseMoveEvent( QMouseEvent* mouseEvent )
                 autoScrollTimer_.start( 100, this );
         }
     }
+    else {
+        considerMouseHovering( mouseEvent->x(), mouseEvent->y() );
+    }
 }
 
 void AbstractLogView::mouseReleaseEvent( QMouseEvent* mouseEvent )
@@ -426,15 +496,36 @@ void AbstractLogView::timerEvent( QTimerEvent* timerEvent )
 void AbstractLogView::keyPressEvent( QKeyEvent* keyEvent )
 {
     LOG(logDEBUG4) << "keyPressEvent received";
+    bool controlModifier = (keyEvent->modifiers() & Qt::ControlModifier) == Qt::ControlModifier;
+    bool shiftModifier = (keyEvent->modifiers() & Qt::ShiftModifier) == Qt::ShiftModifier;
 
     if ( keyEvent->key() == Qt::Key_Left )
         horizontalScrollBar()->triggerAction(QScrollBar::SliderPageStepSub);
     else if ( keyEvent->key() == Qt::Key_Right )
         horizontalScrollBar()->triggerAction(QScrollBar::SliderPageStepAdd);
-    else if ( keyEvent->key() == Qt::Key_Home )
+    else if ( keyEvent->key() == Qt::Key_Home && !controlModifier)
         jumpToStartOfLine();
-    else if ( keyEvent->key() == Qt::Key_End )
+    else if ( keyEvent->key() == Qt::Key_End  && !controlModifier)
         jumpToRightOfScreen();
+    else if ( (keyEvent->key() == Qt::Key_PageDown && controlModifier)
+           || (keyEvent->key() == Qt::Key_End && controlModifier) )
+    {
+        emit followDisabled(); // duplicate of 'G' action.
+        selection_.selectLine( logData->getNbLine() - 1 );
+        emit updateLineNumber( logData->getNbLine() - 1 );
+        jumpToBottom();
+    }
+    else if ( (keyEvent->key() == Qt::Key_PageUp && controlModifier)
+           || (keyEvent->key() == Qt::Key_Home && controlModifier) )
+    {
+        emit followDisabled(); // like 'g' but 0 input first line action.
+        selectAndDisplayLine( 0 );
+        emit updateLineNumber( 0 );
+    }
+    else if ( keyEvent->key() == Qt::Key_F3 && !shiftModifier )
+        searchNext(); // duplicate of 'n' action.
+    else if ( keyEvent->key() == Qt::Key_F3 && shiftModifier )
+        searchPrevious(); // duplicate of 'N' action.
     else {
         const char character = (keyEvent->text())[0].toAscii();
 
@@ -493,10 +584,10 @@ void AbstractLogView::keyPressEvent( QKeyEvent* keyEvent )
                     jumpToBottom();
                     break;
                 case 'n':
-                    searchNext();
+                    emit searchNext();
                     break;
                 case 'N':
-                    searchPrevious();
+                    emit searchPrevious();
                     break;
                 case '*':
                     // Use the selected 'word' and search forward
@@ -545,6 +636,10 @@ void AbstractLogView::scrollContentsBy( int dx, int dy )
     if ( overview_ != NULL )
         overview_->updateCurrentPosition( firstLine, lastLine );
 
+    // Are we hovering over a new line?
+    const QPoint mouse_pos = mapFromGlobal( QCursor::pos() );
+    considerMouseHovering( mouse_pos.x(), mouse_pos.y() );
+
     // Redraw
     update();
 }
@@ -576,6 +671,11 @@ void AbstractLogView::paintEvent( QPaintEvent* paintEvent )
         static const QBrush matchBulletBrush = QBrush( Qt::red );
         static const QBrush markBrush = QBrush( "dodgerblue" );
 
+        static const int SEPARATOR_WIDTH = 1;
+        static const int BULLET_AREA_WIDTH = 11;
+        static const int CONTENT_MARGIN_WIDTH = 1;
+        static const int LINE_NUMBER_PADDING = 3;
+
         // First check the lines to be drawn are within range (might not be the case if
         // the file has just changed)
         const int nbLines = logData->getNbLine();
@@ -594,14 +694,53 @@ void AbstractLogView::paintEvent( QPaintEvent* paintEvent )
 
         // First draw the bullet left margin
         painter.setPen(palette.color(QPalette::Text));
-        painter.drawLine( bulletLineX_, 0, bulletLineX_, viewport()->height() );
-        painter.fillRect( 0, 0, bulletLineX_, viewport()->height(), Qt::darkGray );
+        painter.drawLine( BULLET_AREA_WIDTH, 0,
+                          BULLET_AREA_WIDTH, viewport()->height() );
+        painter.fillRect( 0, 0,
+                          BULLET_AREA_WIDTH, viewport()->height(),
+                          Qt::darkGray );
+
+        // Column at which the content should start (pixels)
+        int contentStartPosX = BULLET_AREA_WIDTH + SEPARATOR_WIDTH;
+
+        // This is also the bullet zone width, used for marking clicks
+        bulletZoneWidthPx_ = contentStartPosX;
+
+        // Draw the line numbers area
+        int lineNumberAreaStartX = 0;
+        if ( lineNumbersVisible_ ) {
+            int lineNumberWidth = charWidth_ * nbDigitsInLineNumber_;
+            int lineNumberAreaWidth =
+                2 * LINE_NUMBER_PADDING + lineNumberWidth;
+            lineNumberAreaStartX = contentStartPosX;
+
+            painter.setPen(palette.color(QPalette::Text));
+            /* Not sure if it looks good...
+            painter.drawLine( contentStartPosX + lineNumberAreaWidth,
+                              0,
+                              contentStartPosX + lineNumberAreaWidth,
+                              viewport()->height() );
+            */
+            painter.fillRect( contentStartPosX, 0,
+                              lineNumberAreaWidth, viewport()->height(),
+                              Qt::lightGray );
+
+            // Update for drawing the actual text
+            contentStartPosX += lineNumberAreaWidth;
+        }
+        else {
+            contentStartPosX += SEPARATOR_WIDTH;
+        }
+
+        // This is the total width of the 'margin' (including line number if any)
+        // used for mouse calculation etc...
+        leftMarginPx_ = contentStartPosX;
 
         // Then draw each line
         for (int i = firstLine; i <= lastLine; i++) {
             // Position in pixel of the base line of the line to print
             const int yPos = (i-firstLine) * fontHeight;
-            const int xPos = bulletLineX_ + 2;
+            const int xPos = contentStartPosX + CONTENT_MARGIN_WIDTH;
 
             // string to print, cut to fit the length and position of the view
             const QString line = lines[i - firstLine];
@@ -693,12 +832,15 @@ void AbstractLogView::paintEvent( QPaintEvent* paintEvent )
                 }
 
                 lineDrawer.draw( painter, xPos, yPos,
-                        viewport()->width(), cutLine );
+                                 viewport()->width(), cutLine,
+                                 CONTENT_MARGIN_WIDTH );
             }
             else {
                 // Nothing to be highlighted, we print the whole line!
-                painter.fillRect( xPos, yPos, viewport()->width(),
-                        fontHeight, backColor );
+                painter.fillRect( xPos - CONTENT_MARGIN_WIDTH, yPos,
+                        viewport()->width(), fontHeight, backColor );
+                // (the rectangle is extended on the left to cover the small
+                // margin, it looks better (LineDrawer does the same) )
                 painter.setPen( foreColor );
                 painter.drawText( xPos, yPos + fontAscent, cutLine );
             }
@@ -707,7 +849,7 @@ void AbstractLogView::paintEvent( QPaintEvent* paintEvent )
             painter.setPen( palette.color( QPalette::Text ) );
             const int circleSize = 3;
             const int arrowHeight = 4;
-            const int middleXLine = bulletLineX_ / 2;
+            const int middleXLine = BULLET_AREA_WIDTH / 2;
             const int middleYLine = yPos + (fontHeight / 2);
 
             const LineType line_type = lineType( i );
@@ -717,7 +859,7 @@ void AbstractLogView::paintEvent( QPaintEvent* paintEvent )
                     QPoint(1, middleYLine - 2),
                     QPoint(middleXLine, middleYLine - 2),
                     QPoint(middleXLine, middleYLine - arrowHeight),
-                    QPoint(bulletLineX_ - 2, middleYLine),
+                    QPoint(BULLET_AREA_WIDTH - 2, middleYLine),
                     QPoint(middleXLine, middleYLine + arrowHeight),
                     QPoint(middleXLine, middleYLine + 2),
                     QPoint(1, middleYLine + 2 ),
@@ -735,50 +877,95 @@ void AbstractLogView::paintEvent( QPaintEvent* paintEvent )
                         middleYLine - circleSize,
                         circleSize * 2, circleSize * 2 );
             }
+
+            // Draw the line number
+            if ( lineNumbersVisible_ ) {
+                static const QString lineNumberFormat( "%1" );
+                const QString& lineNumberStr =
+                    lineNumberFormat.arg( displayLineNumber( i ),
+                                          nbDigitsInLineNumber_ );
+                painter.setPen( palette.color( QPalette::Text ) );
+                painter.drawText( lineNumberAreaStartX + LINE_NUMBER_PADDING,
+                                  yPos + fontAscent, lineNumberStr );
+            }
+
         } // For each line
     }
     LOG(logDEBUG4) << "End of repaint";
 }
 
-void AbstractLogView::setOverview( Overview* overview )
+// These two functions are virtual and this implementation is clearly
+// only valid for a non-filtered display.
+// We count on the 'filtered' derived classes to override them.
+qint64 AbstractLogView::displayLineNumber( int lineNumber ) const
+{
+    return lineNumber + 1; // show a 1-based index
+}
+
+qint64 AbstractLogView::maxDisplayLineNumber() const
+{
+    return logData->getNbLine();
+}
+
+void AbstractLogView::setOverview( Overview* overview,
+       OverviewWidget* overview_widget )
 {
     overview_ = overview;
+    overviewWidget_ = overview_widget;
+
+    if ( overviewWidget_ ) {
+        connect( overviewWidget_, SIGNAL( lineClicked ( int ) ),
+                this, SIGNAL( followDisabled() ) );
+        connect( overviewWidget_, SIGNAL( lineClicked ( int ) ),
+                this, SLOT( jumpToLine( int ) ) );
+    }
     refreshOverview();
-    overviewWidget_.setOverview( overview );
 }
 
-void AbstractLogView::searchNext()
+void AbstractLogView::searchUsingFunction(
+        qint64 (QuickFind::*search_function)() )
 {
-    searchForward();
-}
+    emit followDisabled();
 
-void AbstractLogView::searchPrevious()
-{
-    searchBackward();
+    int line = (quickFind_.*search_function)();
+    if ( line >= 0 ) {
+        LOG(logDEBUG) << "search " << line;
+        displayLine( line );
+        emit updateLineNumber( line );
+    }
 }
 
 void AbstractLogView::searchForward()
 {
-    emit followDisabled();
-
-    int line = quickFind_.searchForward();
-    if ( line >= 0 ) {
-        LOG(logDEBUG) << "searchForward " << line;
-        displayLine( line );
-        emit updateLineNumber( line );
-    }
+    searchUsingFunction( &QuickFind::searchForward );
 }
 
 void AbstractLogView::searchBackward()
 {
-    emit followDisabled();
+    searchUsingFunction( &QuickFind::searchBackward );
+}
 
-    int line = quickFind_.searchBackward();
-    if ( line >= 0 ) {
-        LOG(logDEBUG) << "searchBackward " << line;
-        displayLine( line );
-        emit updateLineNumber( line );
-    }
+void AbstractLogView::incrementallySearchForward()
+{
+    searchUsingFunction( &QuickFind::incrementallySearchForward );
+}
+
+void AbstractLogView::incrementallySearchBackward()
+{
+    searchUsingFunction( &QuickFind::incrementallySearchBackward );
+}
+
+void AbstractLogView::incrementalSearchAbort()
+{
+    quickFind_.incrementalSearchAbort();
+    emit changeQuickFind(
+            "",
+            QuickFindMux::Forward );
+}
+
+void AbstractLogView::incrementalSearchStop()
+{
+    quickFind_.incrementalSearchStop();
 }
 
 void AbstractLogView::followSet( bool checked )
@@ -790,14 +977,16 @@ void AbstractLogView::followSet( bool checked )
 
 void AbstractLogView::refreshOverview()
 {
+    assert( overviewWidget_ );
+
     // Create space for the Overview if needed
     if ( ( getOverview() != NULL ) && getOverview()->isVisible() ) {
         setViewportMargins( 0, 0, OVERVIEW_WIDTH, 0 );
-        overviewWidget_.show();
+        overviewWidget_->show();
     }
     else {
         setViewportMargins( 0, 0, 0, 0 );
-        overviewWidget_.hide();
+        overviewWidget_->hide();
     }
 }
 
@@ -828,8 +1017,9 @@ void AbstractLogView::findNextSelected()
     // Use the selected 'word' and search forward
     if ( selection_.isPortion() ) {
         emit changeQuickFind(
-                selection_.getSelectedText( logData ) );
-        searchNext();
+                selection_.getSelectedText( logData ),
+                QuickFindMux::Forward );
+        emit searchNext();
     }
 }
 
@@ -838,8 +1028,9 @@ void AbstractLogView::findPreviousSelected()
 {
     if ( selection_.isPortion() ) {
         emit changeQuickFind(
-                selection_.getSelectedText( logData ) );
-        searchPrevious();
+                selection_.getSelectedText( logData ),
+                QuickFindMux::Backward );
+        emit searchNext();
     }
 }
 
@@ -888,6 +1079,9 @@ void AbstractLogView::updateData()
     if ( overview_ != NULL )
         overview_->updateCurrentPosition( firstLine, lastLine );
 
+    // Update the length of line numbers
+    nbDigitsInLineNumber_ = countDigits( maxDisplayLineNumber() );
+
     // Repaint!
     update();
 }
@@ -915,8 +1109,10 @@ void AbstractLogView::updateDisplaySize()
     LOG(logDEBUG) << "viewport.height()=" << viewport()->height();
     LOG(logDEBUG) << "width()=" << width();
     LOG(logDEBUG) << "height()=" << height();
-    overviewWidget_.setGeometry( viewport()->width() + 2, 1,
-            OVERVIEW_WIDTH - 1, viewport()->height() );
+
+    if ( overviewWidget_ )
+        overviewWidget_->setGeometry( viewport()->width() + 2, 1,
+                OVERVIEW_WIDTH - 1, viewport()->height() );
 }
 
 int AbstractLogView::getTopLine() const
@@ -929,6 +1125,12 @@ QString AbstractLogView::getSelection() const
     return selection_.getSelectedText( logData );
 }
 
+void AbstractLogView::selectAll()
+{
+    selection_.selectRange( 0, logData->getNbLine() - 1 );
+    update();
+}
+
 void AbstractLogView::selectAndDisplayLine( int line )
 {
     emit followDisabled();
@@ -950,6 +1152,11 @@ void AbstractLogView::jumpToLine( int line )
     verticalScrollBar()->setValue( newTopLine );
 }
 
+void AbstractLogView::setLineNumbersVisible( bool lineNumbersVisible )
+{
+    lineNumbersVisible_ = lineNumbersVisible;
+}
+
 //
 // Private functions
 //
@@ -995,7 +1202,7 @@ QPoint AbstractLogView::convertCoordToFilePos( const QPoint& pos ) const
     if ( column < 0 )
         column = 0;
 
-    LOG(logDEBUG) << "AbstractLogView::convertCoordToFilePos col="
+    LOG(logDEBUG4) << "AbstractLogView::convertCoordToFilePos col="
         << column << " line=" << line;
     QPoint point( column, line );
 
@@ -1192,3 +1399,25 @@ void AbstractLogView::createMenu()
     popupMenu_->addAction( findPreviousAction_ );
     popupMenu_->addAction( addToSearchAction_ );
 }
+
+void AbstractLogView::considerMouseHovering( int x_pos, int y_pos )
+{
+    int line = convertCoordToLine( y_pos );
+    if ( ( x_pos < leftMarginPx_ )
+            && ( line >= 0 )
+            && ( line < logData->getNbLine() ) ) {
+        // Mouse moved in the margin, send event up
+        // (possibly to highlight the overview)
+        if ( line != lastHoveredLine_ ) {
+            LOG(logDEBUG) << "Mouse moved in margin line: " << line;
+            emit mouseHoveredOverLine( line );
+            lastHoveredLine_ = line;
+        }
+    }
+    else {
+        if ( lastHoveredLine_ != -1 ) {
+            emit mouseLeftHoveringZone();
+            lastHoveredLine_ = -1;
+        }
+    }
+}
