@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2009, 2010, 2011 Nicolas Bonnefon and other contributors
+ * Copyright (C) 2009, 2010, 2011, 2012, 2013 Nicolas Bonnefon
+ * and other contributors
  *
  * This file is part of glogg.
  *
@@ -27,6 +28,7 @@
 #include "selection.h"
 #include "quickfind.h"
 #include "overviewwidget.h"
+#include "quickfindmux.h"
 
 class QMenu;
 class QAction;
@@ -76,8 +78,12 @@ class LineDrawer
     // Draw the current line of text using the given painter,
     // in the passed block (in pixels)
     // The line must be cut to fit on the screen.
+    // leftExtraBackgroundPx is the an extra margin to start drawing
+    // the coloured // background, going all the way to the element
+    // left of the line looks better.
     void draw( QPainter& painter, int xPos, int yPos,
-            int line_width, const QString& line );
+               int line_width, const QString& line,
+               int leftExtraBackgroundPx );
 
   private:
     class Chunk {
@@ -137,7 +143,8 @@ class Overview;
 
 // Base class representing the log view widget.
 // It can be either the top (full) or bottom (filtered) view.
-class AbstractLogView : public QAbstractScrollArea
+class AbstractLogView :
+    public QAbstractScrollArea, public SearchableWidgetInterface
 {
   Q_OBJECT
 
@@ -156,8 +163,10 @@ class AbstractLogView : public QAbstractScrollArea
     void updateDisplaySize();
     // Return the line number of the top line of the view
     int getTopLine() const;
-    // Return the line number of the line selected, or -1 if none.
+    // Return the text of the current selection.
     QString getSelection() const;
+    // Instructs the widget to select the whole text.
+    void selectAll();
 
   protected:
     void mousePressEvent( QMouseEvent* mouseEvent );
@@ -177,10 +186,14 @@ class AbstractLogView : public QAbstractScrollArea
     enum LineType { Normal, Marked, Match };
     virtual LineType lineType( int lineNumber ) const = 0;
 
+    // Line number to display for line at the given index
+    virtual qint64 displayLineNumber( int lineNumber ) const;
+    virtual qint64 maxDisplayLineNumber() const;
+
     // Get the overview associated with this view, or NULL if there is none
     Overview* getOverview() const { return overview_; }
-    // Set the Overview
-    void setOverview( Overview* overview );
+    // Set the Overview and OverviewWidget
+    void setOverview( Overview* overview, OverviewWidget* overview_widget );
 
   signals:
     // Sent when a new line has been selected by the user.
@@ -188,11 +201,12 @@ class AbstractLogView : public QAbstractScrollArea
     // Sent up to the MainWindow to disable the follow mode
     void followDisabled();
     // Sent when the view wants the QuickFind widget pattern to change.
-    void changeQuickFind( const QString& newPattern );
+    void changeQuickFind( const QString& newPattern,
+            QuickFindMux::QFDirection newDirection );
     // Sent up when the current line number is updated
     void updateLineNumber( int line );
     // Sent up when quickFind wants to show a message to the user.
-    void notifyQuickFind( const QString& message );
+    void notifyQuickFind( const QFNotification& message );
     // Sent up when quickFind wants to clear the notification.
     void clearQuickFindNotification();
     // Sent when the view ask for a line to be marked
@@ -200,6 +214,13 @@ class AbstractLogView : public QAbstractScrollArea
     void markLine( qint64 line );
     // Sent up when the user wants to add the selection to the search
     void addToSearch( const QString& selection );
+    // Sent up when the mouse is hovered over a line's margin
+    void mouseHoveredOverLine( qint64 line );
+    // Sent up when the mouse leaves a line's margin
+    void mouseLeftHoveringZone();
+    // Sent up for view initiated quickfind searches
+    void searchNext();
+    void searchPrevious();
 
   public slots:
     // Makes the widget select and display the passed line.
@@ -207,9 +228,18 @@ class AbstractLogView : public QAbstractScrollArea
     void selectAndDisplayLine( int line );
 
     // Use the current QFP to go and select the next match.
-    void searchForward();
+    virtual void searchForward();
     // Use the current QFP to go and select the previous match.
-    void searchBackward();
+    virtual void searchBackward();
+
+    // Use the current QFP to go and select the next match (incremental)
+    virtual void incrementallySearchForward();
+    // Use the current QFP to go and select the previous match (incremental)
+    virtual void incrementallySearchBackward();
+    // Stop the current incremental search (typically when user press return)
+    virtual void incrementalSearchStop();
+    // Abort the current incremental search (typically when user press esc)
+    virtual void incrementalSearchAbort();
 
     // Signals the follow mode has been enabled.
     void followSet( bool checked );
@@ -222,6 +252,9 @@ class AbstractLogView : public QAbstractScrollArea
     // (does NOT emit followDisabled() )
     void jumpToLine( int line );
 
+    // Configure the setting of whether to show line number margin
+    void setLineNumbersVisible( bool lineNumbersVisible );
+
   private slots:
     void handlePatternUpdated();
     void addToSearch();
@@ -231,22 +264,37 @@ class AbstractLogView : public QAbstractScrollArea
 
   private:
     // Constants
-    static const int bulletLineX_;
-    static const int leftMarginPx_;
     static const int OVERVIEW_WIDTH;
 
-    // Digits buffer
+    // Width of the bullet zone, including decoration
+    int bulletZoneWidthPx_;
+
+    // Total size of all margins and decorations in pixels
+    int leftMarginPx_;
+
+    // Number of digits to display in line numbers
+    int nbDigitsInLineNumber_;
+
+    // Digits buffer (for numeric keyboard entry)
     DigitsBuffer digitsBuffer_;
 
     // Follow mode
     bool followMode_;
 
+    // Whether to show line numbers or not
+    bool lineNumbersVisible_;
+
     // Pointer to the CrawlerWidget's data set
     const AbstractLogData* logData;
 
     // Pointer to the Overview object
     Overview* overview_;
 
+    // Pointer to the OverviewWidget, this class doesn't own it,
+    // but is responsible for displaying it (for purely aesthetic
+    // reasons).
+    OverviewWidget* overviewWidget_;
+
     bool selectionStarted_;
     // Start of the selection (characters)
     QPoint selectionStartPos_;
@@ -254,6 +302,10 @@ class AbstractLogView : public QAbstractScrollArea
     QPoint selectionCurrentEndPos_;
     QBasicTimer autoScrollTimer_;
 
+    // Hovering state
+    // Last line that has been hoovered on, -1 if none
+    qint64 lastHoveredLine_;
+
     // Marks (left margin click)
     bool markingClickInitiated_;
     qint64 markingClickLine_;
@@ -279,12 +331,8 @@ class AbstractLogView : public QAbstractScrollArea
     // Our own QuickFind object
     QuickFind quickFind_;
 
-    OverviewWidget overviewWidget_;
-
     int getNbVisibleLines() const;
     int getNbVisibleCols() const;
-    void convertCoordToFilePos( const QPoint& pos,
-            int* line, int* column ) const;
     QPoint convertCoordToFilePos( const QPoint& pos ) const;
     int convertCoordToLine( int yPos ) const;
     int convertCoordToColumn( int xPos ) const;
@@ -299,9 +347,10 @@ class AbstractLogView : public QAbstractScrollArea
 
     void createMenu();
 
+    void considerMouseHovering( int x_pos, int y_pos );
+
     // Search functions (for n/N)
-    void searchNext();
-    void searchPrevious();
+    void searchUsingFunction ( qint64 (QuickFind::*search_function)() );
 
     // Utils functions
     bool isCharWord( char c );
