@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009, 2010, 2011 Nicolas Bonnefon and other contributors
+ * Copyright (C) 2009, 2010, 2011, 2013 Nicolas Bonnefon and other contributors
  *
  * This file is part of glogg.
  *
@@ -46,16 +46,28 @@ class Configuration : public Persistable {
     { return mainRegexpType_; }
     SearchRegexpType quickfindRegexpType() const
     { return quickfindRegexpType_; }
+    bool isQuickfindIncremental() const
+    { return quickfindIncremental_; }
     void setMainRegexpType( SearchRegexpType type )
     { mainRegexpType_ = type; }
     void setQuickfindRegexpType( SearchRegexpType type )
     { quickfindRegexpType_ = type; }
+    void setQuickfindIncremental( bool is_incremental )
+    { quickfindIncremental_ = is_incremental; }
 
     // View settings
     bool isOverviewVisible() const
     { return overviewVisible_; }
     void setOverviewVisible( bool isVisible )
     { overviewVisible_ = isVisible; }
+    bool mainLineNumbersVisible() const
+    { return lineNumbersVisibleInMain_; }
+    bool filteredLineNumbersVisible() const
+    { return lineNumbersVisibleInFiltered_; }
+    void setMainLineNumbersVisible( bool lineNumbersVisible )
+    { lineNumbersVisibleInMain_ = lineNumbersVisible; }
+    void setFilteredLineNumbersVisible( bool lineNumbersVisible )
+    { lineNumbersVisibleInFiltered_ = lineNumbersVisible; }
 
     // Reads/writes the current config in the QSettings object passed
     virtual void saveToStorage( QSettings& settings ) const;
@@ -66,9 +78,12 @@ class Configuration : public Persistable {
     QFont mainFont_;
     SearchRegexpType mainRegexpType_;
     SearchRegexpType quickfindRegexpType_;
+    bool quickfindIncremental_;
 
     // View settings
     bool overviewVisible_;
+    bool lineNumbersVisibleInMain_;
+    bool lineNumbersVisibleInFiltered_;
 };
 
 #endif
