@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009, 2010, 2011, 2012 Nicolas Bonnefon and other contributors
+ * Copyright (C) 2009, 2010, 2011, 2012, 2013 Nicolas Bonnefon and other contributors
  *
  * This file is part of glogg.
  *
@@ -57,20 +57,29 @@ CrawlerWidget::CrawlerWidget(SavedSearches* searches, QWidget *parent)
     logData_          = new LogData();
     logFilteredData_  = logData_->getNewFilteredData();
 
-    // Initialise the QFP object (one for both views)
-    // This is the confirmed pattern used by n/N and coloured in yellow.
-    quickFindPattern_ = new QuickFindPattern();
-
     // The matches overview
     overview_ = new Overview();
 
+    // Initialise the QF Mux to send requests from the QFWidget
+    // to the right window
+    quickFindMux_ = new QuickFindMux( this );
+
     // The views
-    logMainView  = new LogMainView( logData_, quickFindPattern_, overview_ );
     bottomWindow = new QWidget;
-    filteredView = new FilteredView( logFilteredData_, quickFindPattern_ );
+    overviewWidget_ = new OverviewWidget();
+    logMainView     = new LogMainView(
+            logData_, quickFindMux_->getPattern(), overview_, overviewWidget_ );
+    filteredView    = new FilteredView(
+            logFilteredData_, quickFindMux_->getPattern() );
+
+    overviewWidget_->setOverview( overview_ );
+    overviewWidget_->setParent( logMainView );
 
     savedSearches = searches;
 
+    quickFindMux_->registerSearchable( logMainView );
+    quickFindMux_->registerSearchable( filteredView );
+
     // Construct the visibility button
     visibilityModel_ = new QStandardItemModel( this );
 
@@ -183,8 +192,8 @@ CrawlerWidget::CrawlerWidget(SavedSearches* searches, QWidget *parent)
     bottomMainLayout->setContentsMargins(2, 1, 2, 1);
     bottomWindow->setLayout(bottomMainLayout);
 
-    addWidget(logMainView);
-    addWidget(bottomWindow);
+    addWidget( logMainView );
+    addWidget( bottomWindow );
 
     // Default splitter position (usually overridden by the config file)
     QList<int> splitterSizes;
@@ -223,38 +232,47 @@ CrawlerWidget::CrawlerWidget(SavedSearches* searches, QWidget *parent)
     connect(filteredView, SIGNAL( addToSearch( const QString& ) ),
             this, SLOT( addToSearch( const QString& ) ) );
 
+    connect(filteredView, SIGNAL( mouseHoveredOverLine( qint64 ) ),
+            this, SLOT( mouseHoveredOverMatch( qint64 ) ) );
+    connect(filteredView, SIGNAL( mouseLeftHoveringZone() ),
+            overviewWidget_, SLOT( removeHighlight() ) );
+
     // Follow option (up and down)
     connect(this, SIGNAL( followSet( bool ) ),
             logMainView, SLOT( followSet( bool ) ) );
+    connect(this, SIGNAL( followSet( bool ) ),
+            filteredView, SLOT( followSet( bool ) ) );
     connect(logMainView, SIGNAL( followDisabled() ),
             this, SIGNAL( followDisabled() ) );
+    connect(filteredView, SIGNAL( followDisabled() ),
+            this, SIGNAL( followDisabled() ) );
 
     connect( logFilteredData_, SIGNAL( searchProgressed( int, int ) ),
             this, SLOT( updateFilteredView( int, int ) ) );
 
     // QuickFind
-    connect( quickFindWidget_, SIGNAL( patternConfirmed( const QString& ) ),
-            this, SLOT( applyNewQFPattern( const QString& ) ) );
     connect( quickFindWidget_, SIGNAL( close() ),
-            this, SLOT( hideQuickFindBar() ) );
+             this, SLOT( hideQuickFindBar() ) );
+    connect( quickFindWidget_, SIGNAL( patternConfirmed( const QString&, bool ) ),
+             quickFindMux_, SLOT( confirmPattern( const QString&, bool ) ) );
+    connect( quickFindWidget_, SIGNAL( patternUpdated( const QString&, bool ) ),
+             quickFindMux_, SLOT( setNewPattern( const QString&, bool ) ) );
+    connect( quickFindWidget_, SIGNAL( cancelSearch() ),
+             quickFindMux_, SLOT( cancelSearch() ) );
     connect( quickFindWidget_, SIGNAL( searchForward() ),
-            this, SLOT( searchForward() ) );
+             quickFindMux_, SLOT( searchForward() ) );
     connect( quickFindWidget_, SIGNAL( searchBackward() ),
-            this, SLOT( searchBackward() ) );
+             quickFindMux_, SLOT( searchBackward() ) );
+    connect( quickFindWidget_, SIGNAL( searchNext() ),
+             quickFindMux_, SLOT( searchNext() ) );
 
     // QuickFind changes coming from the views
-    connect(logMainView, SIGNAL( changeQuickFind( const QString& ) ),
-            this, SLOT( changeQFPattern( const QString& ) ) );
-    connect(filteredView, SIGNAL( changeQuickFind( const QString& ) ),
-            this, SLOT( changeQFPattern( const QString& ) ) );
-    connect(logMainView, SIGNAL( notifyQuickFind( const QString& ) ),
-            quickFindWidget_, SLOT( notify( const QString& ) ) );
-    connect(filteredView, SIGNAL( notifyQuickFind( const QString& ) ),
-            quickFindWidget_, SLOT( notify( const QString& ) ) );
-    connect(logMainView, SIGNAL( clearQuickFindNotification() ),
-            quickFindWidget_, SLOT( clearNotification() ) );
-    connect(filteredView, SIGNAL( clearQuickFindNotification() ),
-            quickFindWidget_, SLOT( clearNotification() ) );
+    connect( quickFindMux_, SIGNAL( patternChanged( const QString& ) ),
+             this, SLOT( changeQFPattern( const QString& ) ) );
+    connect( quickFindMux_, SIGNAL( notify( const QFNotification& ) ),
+             quickFindWidget_, SLOT( notify( const QFNotification& ) ) );
+    connect( quickFindMux_, SIGNAL( clearNotification() ),
+             quickFindWidget_, SLOT( clearNotification() ) );
 
     // Sent load file update to MainWindow (for status update)
     connect( logData_, SIGNAL( loadingProgressed( int ) ),
@@ -325,6 +343,29 @@ QString CrawlerWidget::getSelectedText() const
         return logMainView->getSelection();
 }
 
+void CrawlerWidget::selectAll()
+{
+    activeView()->selectAll();
+}
+
+// Return a pointer to the view in which we should do the QuickFind
+SearchableWidgetInterface* CrawlerWidget::getActiveSearchable() const
+{
+    QWidget* searchableWidget;
+
+    // Search in the window that has focus, or the window where 'Find' was
+    // called from, or the main window.
+    if ( filteredView->hasFocus() || logMainView->hasFocus() )
+        searchableWidget = QApplication::focusWidget();
+    else
+        searchableWidget = qfSavedFocus_;
+
+    if ( AbstractLogView* view = qobject_cast<AbstractLogView*>( searchableWidget ) )
+        return view;
+    else
+        return logMainView;
+}
+
 //
 // Events handlers
 //
@@ -335,8 +376,10 @@ void CrawlerWidget::keyPressEvent( QKeyEvent* keyEvent )
 
     switch ( (keyEvent->text())[0].toAscii() ) {
         case '/':
+            displayQuickFindBar( QuickFindMux::Forward );
+            break;
         case '?':
-            displayQuickFindBar();
+            displayQuickFindBar( QuickFindMux::Backward );
             break;
         default:
             keyEvent->ignore();
@@ -465,6 +508,9 @@ void CrawlerWidget::applyConfiguration()
     logMainView->setFont(font);
     filteredView->setFont(font);
 
+    logMainView->setLineNumbersVisible( config.mainLineNumbersVisible() );
+    filteredView->setLineNumbersVisible( config.filteredLineNumbersVisible() );
+
     overview_->setVisible( config.isOverviewVisible() );
     logMainView->refreshOverview();
 
@@ -514,13 +560,14 @@ void CrawlerWidget::fileChangedHandler( LogData::MonitoredFileStatus status )
     }
 }
 
-void CrawlerWidget::displayQuickFindBar()
+void CrawlerWidget::displayQuickFindBar( QuickFindMux::QFDirection direction )
 {
     LOG(logDEBUG) << "CrawlerWidget::displayQuickFindBar";
 
     // Remember who had the focus
     qfSavedFocus_ = QApplication::focusWidget();
 
+    quickFindMux_->setDirection( direction );
     quickFindWidget_->userActivate();
 }
 
@@ -530,30 +577,24 @@ void CrawlerWidget::hideQuickFindBar()
     qfSavedFocus_->setFocus();
 }
 
-void CrawlerWidget::applyNewQFPattern( const QString& newPattern )
-{
-    quickFindPattern_->changeSearchPattern( newPattern );
-}
-
 void CrawlerWidget::changeQFPattern( const QString& newPattern )
 {
     quickFindWidget_->changeDisplayedPattern( newPattern );
-    quickFindPattern_->changeSearchPattern( newPattern );
 }
 
 // Returns a pointer to the window in which the search should be done
-AbstractLogView* CrawlerWidget::searchableWidget() const
+AbstractLogView* CrawlerWidget::activeView() const
 {
-    QWidget* searchableWidget;
+    QWidget* activeView;
 
     // Search in the window that has focus, or the window where 'Find' was
     // called from, or the main window.
     if ( filteredView->hasFocus() || logMainView->hasFocus() )
-        searchableWidget = QApplication::focusWidget();
+        activeView = QApplication::focusWidget();
     else
-        searchableWidget = qfSavedFocus_;
+        activeView = qfSavedFocus_;
 
-    if ( AbstractLogView* view = qobject_cast<AbstractLogView*>( searchableWidget ) )
+    if ( AbstractLogView* view = qobject_cast<AbstractLogView*>( activeView ) )
         return view;
     else
         return logMainView;
@@ -563,14 +604,14 @@ void CrawlerWidget::searchForward()
 {
     LOG(logDEBUG) << "CrawlerWidget::searchForward";
 
-    searchableWidget()->searchForward();
+    activeView()->searchForward();
 }
 
 void CrawlerWidget::searchBackward()
 {
     LOG(logDEBUG) << "CrawlerWidget::searchBackward";
 
-    searchableWidget()->searchBackward();
+    activeView()->searchBackward();
 }
 
 void CrawlerWidget::searchRefreshChangedHandler( int state )
@@ -612,6 +653,13 @@ void CrawlerWidget::addToSearch( const QString& string )
     searchLineEdit->lineEdit()->setFocus();
 }
 
+void CrawlerWidget::mouseHoveredOverMatch( qint64 line )
+{
+    qint64 line_in_mainview = logFilteredData_->getMatchingLineNumber( line );
+
+    overviewWidget_->highlightLine( line_in_mainview );
+}
+
 //
 // Private functions
 //
