@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2009, 2010, 2011 Nicolas Bonnefon and other contributors
+ * Copyright (C) 2009, 2010, 2011, 2013 Nicolas Bonnefon
+ * and other contributors
  *
  * This file is part of glogg.
  *
@@ -31,9 +32,10 @@
 
 #include "logmainview.h"
 #include "filteredview.h"
-
 #include "logdata.h"
 #include "logfiltereddata.h"
+#include "quickfindwidget.h"
+#include "quickfindmux.h"
 
 class InfoLine;
 class QuickFindPattern;
@@ -41,11 +43,12 @@ class QuickFindWidget;
 class SavedSearches;
 class Overview;
 class QStandardItemModel;
+class OverviewWidget;
 
 // Implements the central widget of the application.
 // It includes both windows, the search line, the info
 // lines and various buttons.
-class CrawlerWidget : public QSplitter
+class CrawlerWidget : public QSplitter, public QuickFindMuxSelectorInterface
 {
   Q_OBJECT
 
@@ -65,7 +68,15 @@ class CrawlerWidget : public QSplitter
     QString getSelectedText() const;
 
     // Display the QFB at the bottom, remembering where the focus was
-    void displayQuickFindBar();
+    void displayQuickFindBar( QuickFindMux::QFDirection direction );
+
+    // Instructs the widget to select all the text in the window the user
+    // is interacting with
+    void selectAll();
+
+    // Implementation on the mux selector interface
+    // (for dispatching QuickFind to the right widget)
+    virtual SearchableWidgetInterface* getActiveSearchable() const;
 
   protected:
     void keyPressEvent( QKeyEvent* keyEvent );
@@ -107,9 +118,6 @@ class CrawlerWidget : public QSplitter
 
     void hideQuickFindBar();
 
-    // Instructs the widget to apply the pattern (called by the QF widget).
-    void applyNewQFPattern( const QString& newPattern );
-
     // Instructs the widget to change the pattern in the QuickFind widget
     // and confirm it.
     void changeQFPattern( const QString& newPattern );
@@ -129,6 +137,9 @@ class CrawlerWidget : public QSplitter
     // Called when the user add the string to the search
     void addToSearch( const QString& string );
 
+    // Called when a match is hovered on in the filtered view
+    void mouseHoveredOverMatch( qint64 line );
+
   private:
     // State machine holding the state of the search, used to allow/disallow
     // auto-refresh and inform the user via the info line.
@@ -170,7 +181,7 @@ class CrawlerWidget : public QSplitter
     // Private functions
     void replaceCurrentSearch( const QString& searchText );
     void updateSearchCombo();
-    AbstractLogView* searchableWidget() const;
+    AbstractLogView* activeView() const;
     void printSearchInfoMessage( int nbMatches = 0 );
 
     // Palette for error notification (yellow background)
@@ -188,6 +199,7 @@ class CrawlerWidget : public QSplitter
     QCheckBox*      ignoreCaseCheck;
     QCheckBox*      searchRefreshCheck;
     QuickFindWidget* quickFindWidget_;
+    OverviewWidget* overviewWidget_;
 
     QVBoxLayout*    bottomMainLayout;
     QHBoxLayout*    searchLineLayout;
@@ -198,7 +210,7 @@ class CrawlerWidget : public QSplitter
 
     SavedSearches*  savedSearches;
 
-    QuickFindPattern* quickFindPattern_;
+    QuickFindMux*   quickFindMux_;
 
     LogData*        logData_;
     LogFilteredData* logFilteredData_;
