@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009, 2010 Nicolas Bonnefon and other contributors
+ * Copyright (C) 2009, 2010, 2011 Nicolas Bonnefon and other contributors
  *
  * This file is part of glogg.
  *
@@ -31,6 +31,7 @@ using namespace std;
 #include "filterset.h"
 #include "recentfiles.h"
 #include "mainwindow.h"
+#include "savedsearches.h"
 #include "log.h"
 
 static void print_version();
@@ -115,6 +116,7 @@ int main(int argc, char *argv[])
     FILELog::setReportingLevel( logLevel );
 
     // Register the configuration items
+    GetPersistentInfo().migrateAndInit();
     GetPersistentInfo().registerPersistable(
             new SessionInfo, QString( "session" ) );
     GetPersistentInfo().registerPersistable(
@@ -126,6 +128,9 @@ int main(int argc, char *argv[])
     GetPersistentInfo().registerPersistable(
             new RecentFiles, QString( "recentFiles" ) );
 
+    // FIXME: should be replaced by a two staged init of MainWindow
+    GetPersistentInfo().retrieve( QString( "settings" ) );
+
     MainWindow* mw = new MainWindow();
 
     LOG(logDEBUG) << "MainWindow created.";
