@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009, 2010, 2011 Nicolas Bonnefon and other contributors
+ * Copyright (C) 2009, 2010, 2011, 2013 Nicolas Bonnefon and other contributors
  *
  * This file is part of glogg.
  *
@@ -139,6 +139,12 @@ void MainWindow::createActions()
     copyAction->setStatusTip(tr("Copy the selection"));
     connect( copyAction, SIGNAL(triggered()), this, SLOT(copy()) );
 
+    selectAllAction = new QAction(tr("Select &All"), this);
+    selectAllAction->setShortcut(tr("Ctrl+A"));
+    selectAllAction->setStatusTip(tr("Select all the text"));
+    connect( selectAllAction, SIGNAL(triggered()),
+             this, SLOT( selectAll() ) );
+
     findAction = new QAction(tr("&Find..."), this);
     findAction->setShortcut(QKeySequence::Find);
     findAction->setStatusTip(tr("Find the text"));
@@ -151,6 +157,20 @@ void MainWindow::createActions()
     connect( overviewVisibleAction, SIGNAL( toggled( bool ) ),
             this, SLOT( toggleOverviewVisibility( bool )) );
 
+    lineNumbersVisibleInMainAction =
+        new QAction( tr("Line &numbers in main view"), this );
+    lineNumbersVisibleInMainAction->setCheckable( true );
+    lineNumbersVisibleInMainAction->setChecked( config.mainLineNumbersVisible() );
+    connect( lineNumbersVisibleInMainAction, SIGNAL( toggled( bool ) ),
+            this, SLOT( toggleMainLineNumbersVisibility( bool )) );
+
+    lineNumbersVisibleInFilteredAction =
+        new QAction( tr("Line &numbers in filtered view"), this );
+    lineNumbersVisibleInFilteredAction->setCheckable( true );
+    lineNumbersVisibleInFilteredAction->setChecked( config.filteredLineNumbersVisible() );
+    connect( lineNumbersVisibleInFilteredAction, SIGNAL( toggled( bool ) ),
+            this, SLOT( toggleFilteredLineNumbersVisibility( bool )) );
+
     followAction = new QAction( tr("&Follow File"), this );
     followAction->setShortcut(Qt::Key_F);
     followAction->setCheckable(true);
@@ -199,11 +219,16 @@ void MainWindow::createMenus()
 
     editMenu = menuBar()->addMenu( tr("&Edit") );
     editMenu->addAction( copyAction );
+    editMenu->addAction( selectAllAction );
+    editMenu->addSeparator();
     editMenu->addAction( findAction );
 
     viewMenu = menuBar()->addMenu( tr("&View") );
     viewMenu->addAction( overviewVisibleAction );
     viewMenu->addSeparator();
+    viewMenu->addAction( lineNumbersVisibleInMainAction );
+    viewMenu->addAction( lineNumbersVisibleInFilteredAction );
+    viewMenu->addSeparator();
     viewMenu->addAction( followAction );
     viewMenu->addSeparator();
     viewMenu->addAction( reloadAction );
@@ -270,6 +295,12 @@ void MainWindow::openRecentFile()
         loadFile(action->data().toString());
 }
 
+// Select all the text in the currently selected view
+void MainWindow::selectAll()
+{
+    crawlerWidget->selectAll();
+}
+
 // Copy the currently selected line into the clipboard
 void MainWindow::copy()
 {
@@ -282,9 +313,10 @@ void MainWindow::copy()
             QClipboard::Selection );
 }
 
+// Display the QuickFind bar
 void MainWindow::find()
 {
-    crawlerWidget->displayQuickFindBar();
+    crawlerWidget->displayQuickFindBar( QuickFindMux::Forward );
 }
 
 // Reload the current log file
@@ -325,7 +357,7 @@ void MainWindow::about()
 #ifdef GLOGG_COMMIT
                 "<p>Built " GLOGG_DATE " from " GLOGG_COMMIT
 #endif
-                "<p>Copyright &copy; 2009, 2010, 2011 2012 Nicolas Bonnefon and other contributors"
+                "<p>Copyright &copy; 2009, 2010, 2011, 2012, 2013 Nicolas Bonnefon and other contributors"
                 "<p>You may modify and redistribute the program under the terms of the GPL (version 3 or later)." ) );
 }
 
@@ -337,9 +369,21 @@ void MainWindow::aboutQt()
 void MainWindow::toggleOverviewVisibility( bool isVisible )
 {
     Configuration& config = Persistent<Configuration>( "settings" );
-
     config.setOverviewVisible( isVisible );
+    emit optionsChanged();
+}
 
+void MainWindow::toggleMainLineNumbersVisibility( bool isVisible )
+{
+    Configuration& config = Persistent<Configuration>( "settings" );
+    config.setMainLineNumbersVisible( isVisible );
+    emit optionsChanged();
+}
+
+void MainWindow::toggleFilteredLineNumbersVisibility( bool isVisible )
+{
+    Configuration& config = Persistent<Configuration>( "settings" );
+    config.setFilteredLineNumbersVisible( isVisible );
     emit optionsChanged();
 }
 
