@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009, 2010, 2011 Nicolas Bonnefon and other contributors
+ * Copyright (C) 2009, 2010, 2011, 2013 Nicolas Bonnefon and other contributors
  *
  * This file is part of glogg.
  *
@@ -49,6 +49,7 @@ class MainWindow : public QMainWindow
   private slots:
     void open();
     void openRecentFile();
+    void selectAll();
     void copy();
     void find();
     void reload();
@@ -60,6 +61,8 @@ class MainWindow : public QMainWindow
 
     // Change the view settings
     void toggleOverviewVisibility( bool isVisible );
+    void toggleMainLineNumbersVisibility( bool isVisible );
+    void toggleFilteredLineNumbersVisibility( bool isVisible );
 
     // Disable the follow mode checkbox and send the followSet signal down
     void disableFollow();
@@ -123,8 +126,11 @@ class MainWindow : public QMainWindow
     QAction *openAction;
     QAction *exitAction;
     QAction *copyAction;
+    QAction *selectAllAction;
     QAction *findAction;
     QAction *overviewVisibleAction;
+    QAction *lineNumbersVisibleInMainAction;
+    QAction *lineNumbersVisibleInFilteredAction;
     QAction *followAction;
     QAction *reloadAction;
     QAction *stopAction;
