@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011 Nicolas Bonnefon and other contributors
+ * Copyright (C) 2011, 2012, 2013 Nicolas Bonnefon and other contributors
  *
  * This file is part of glogg.
  *
@@ -21,6 +21,7 @@
 #define OVERVIEWWIDGET_H
 
 #include <QWidget>
+#include <QBasicTimer>
 
 class Overview;
 
@@ -29,15 +30,22 @@ class OverviewWidget : public QWidget
   Q_OBJECT
 
   public:
-    OverviewWidget( QWidget* parent );
+    OverviewWidget( QWidget* parent = 0 );
 
     // Associate the widget with an Overview object.
     void setOverview( Overview* overview ) { overview_ = overview; }
 
+  public slots:
+    // Sent when a match at the line passed must be highlighted in
+    // the overview
+    void highlightLine( qint64 line );
+    void removeHighlight();
+
   protected:
     void paintEvent( QPaintEvent* paintEvent );
     void mousePressEvent( QMouseEvent* mouseEvent );
     void mouseMoveEvent( QMouseEvent* mouseEvent );
+    void timerEvent( QTimerEvent* event );
 
   signals:
     // Sent when the user click on a line in the Overview.
@@ -46,9 +54,19 @@ class OverviewWidget : public QWidget
   private:
     // Constants
     static const int LINE_MARGIN;
+    static const int STEP_DURATION_MS;
+    static const int INITIAL_TTL_VALUE;
 
     Overview* overview_;
 
+    // Highlight:
+    // Which line is higlighted, or -1 if none
+    int highlightedLine_;
+    // Number of step until the highlight become static
+    int highlightedTTL_;
+
+    QBasicTimer highlightTimer_;
+
     void handleMousePress( int position );
 };
 
